import sys
import pathlib
import struct

import numpy.ma


def get_data_file_path(file_name):
    file_path = pathlib.Path(file_name)
    if file_path.is_absolute():
        return file_path.resolve()
    else:
        # 相対PATHだった時.
        # venv etc.
        file_path_venv = (pathlib.Path(sys.prefix) / "data" / file_name).resolve()
        if file_path_venv.exists():
            return file_path_venv
        # develop
        file_path_dev = (pathlib.Path.cwd() / "data" / file_name).resolve()
        if file_path_dev.exists():
            print("dev:", file_path_dev)
            return file_path_dev.resolve()
        else:
            print(file_path)
            raise OSError("wav file path not exist")


# 変換用関数群
def normalization(array) -> list:
    # エフェクトをかけやすいようにバイナリデータを[-1, +1]に正規化
    # ndArray -> ndArray
    # int16 の絶対値は 32767
    return numpy.ma.frombuffer(array, dtype="int16") / 32768.0


def de_normalization(array) -> bytes:
    # 正規化前のバイナリデータに戻す(32768倍)
    new_data = [int(x * 32767.0) for x in array]
    return struct.pack("h" * len(new_data), *new_data)



from pathlib import Path
import numpy as np
from otoware import convert_wav_and_array as cwa, utils


class DistortionWavAndArray(cwa.WavAndArray):
    def __init__(self, origin_path: Path, result_path: Path = None,
                 distortion_level=20):
        super(DistortionWavAndArray, self).__init__(origin_path)
        self._dist_level = distortion_level

    def bytes_data_generator(self):
        # 正規化
        self._data = utils.normalization(self._data)
        frame_per_buffer = 1024
        position = 0
        size = len(self._data)
        while position < size:
            yield utils.de_normalization(distortion(
                self._data[position:position + frame_per_buffer],
                self._dist_level))
            position += frame_per_buffer

    def update(self, distortion_level=20):
        """今回の肝
        このクラスのself._dist_levelを変更したらリアルタイムで音割れ度が変わる
        """
        self._dist_level = distortion_level


# byteもしくはbyte列に対して distortion の処理をして送る
def distortion(data, gain, level=0.7, clip=True):
    """gain乗の値をもちいてdistortion
    data: numpy.ndarray
    gain: 増幅の倍率
    level: 音量
    clip: ハードクリッピングするか(下記Qiitaか論文参照)
    """
    # https://qiita.com/stringamp/items/4b6e344ddf878f5099c7#122-%E5%AE%9F%E8%A3%85
    # 単純にgain倍増幅する時は以下
    # for n in range(length):
    # new_data = data * gain
    new_data = np.sign(data) * (1 - np.exp(-1 * gain * np.abs(data)))
    if clip:
        new_data = np.clip(new_data, -1.0, 1.0)
    # 音量を調整
    new_data *= level
    return new_data



import argparse

from otoware.distortion import DistortionWavAndArray
from otoware.utils import get_data_file_path


def main():
    parser = argparse.ArgumentParser(description='このプログラムの説明（なくてもよい）')  # 2. パーサを作る

    # 3. parser.add_argumentで受け取る引数を追加していく
    parser.add_argument('-d', '--dist_level', help='distortion level', type=int, default=1024)
    parser.add_argument('-f', '--file_name', help='wav file name', default="origin.wav")

    args = parser.parse_args()
    # 多分これと別にThreadを立ててotowari_levelを更新させればいい
    play_otowari(args.file_name, args.dist_level)


def play_otowari(file_name, distortion_level=20):
    origin_path = get_data_file_path(file_name)
    # wav_file=WavAndArray(origin_path)
    wav_file = DistortionWavAndArray(origin_path,
                                     distortion_level=distortion_level)
    wav_file.play_file()


if __name__ == '__main__':
    main()



import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.md')) as f:
    README = f.read()

requires = [
    'numpy',
    'matplotlib',
    'pyaudio'
]

tests_require = [
    'pytest',
    'pytest-cov',
]

setup(name='otoware',
      version='1.1.1',
      description='otoware',
      long_description=README,
      long_description_content_type="text/markdown",
      classifiers=[
          "Programming Language :: Python",
      ],
      author='kkiyama117',
      author_email='k.kiyama117@gmail.com',
      url='https://ku-jinja.net',
      keywords='python',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires=requires,
      extras_require={
          'testing': tests_require,
      },
      data_files=[('data', ['data/origin.wav'])],
      entry_points={
          'console_scripts': [
              'otowari = otoware.api:main',
          ],
      },
      )



import wave
import pyaudio
from pathlib import Path


class WavAndArray:
    def __init__(self, origin_path: Path, result_path: Path = None):
        self._path: Path = origin_path
        self._result_path = result_path
        # 元のWavファイルから取得した音楽データ情報
        self._wav_info = self.get_wave_info()
        # ndarray化した音楽データ
        self._data = self.wav_to_ndarray()
        print(self._wav_info)

    # 初期化関数群
    def get_wave_info(self):
        """WAVEファイルの情報を取得"""
        with self._path.open("rb") as f, wave.open(f) as wf:
            return {'channel': wf.getnchannels(), 'width': wf.getsampwidth(),
                    'frame_rate': wf.getframerate(), 'frames': wf.getnframes(),
                    'params': wf.getparams(),
                    "long": float(wf.getnframes() / wf.getframerate())
                    }

    def wav_to_ndarray(self):
        with self._path.open("rb") as f, wave.open(f) as wf:
            return wf.readframes(self._wav_info['frames'])

    # 再生用関数群
    # byte列を(本来のwavのデータの情報を元に再生)
    def play_file(self):
        p = pyaudio.PyAudio()
        # Streamを生成(3)
        stream = p.open(
            format=p.get_format_from_width(self._wav_info['width']),
            channels=self._wav_info['channel'],
            rate=self._wav_info['frame_rate'],
            output=True)
        for byte_data in self.bytes_data_generator():
            stream.write(byte_data)
        stream.close()
        p.terminate()

    # byte列の末端になるまで1フレーム毎にデータを出力
    def bytes_data_generator(self):
        frame_per_buffer = 1024
        position = 0
        size = len(self._data)
        while position < size:
            yield self._data[position:position + frame_per_buffer]
            position += frame_per_buffer


# """保存用関数群"""
#
#
# array をwav に変換して保存
# def save_file(bit_array, origin_path: Path, result_path: Path):
#     音声を保存
#     wav_info = get_wave_info(origin_path)
#     frame_rate = wav_info['frame_rate']
#     channel = wav_info['channel']
#     ndarray_to_wav(bit_array, channel=channel, rate=frame_rate,
#                    path=result_path)
#
#
# def ndarray_to_wav(data: bytes, channel: int, rate: int, path: Path):
#     """波形データをWAVEファイルへ出力"""
#     with path.open("wb") as f, wave.open(f, "w") as wf:
#         wf.setnchannels(channel)
#         16bit // 2 で サンプルサイズは2(らしい)
# wf.setsampwidth(2)
# wf.setframerate(rate)
# wf.writeframes(data)
# print("data completely saved")
# print(get_wave_info(path=path))
